﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// VMdemoWPF
// Example of an approach to ViewModel for scientific software, using WPF
// Released for educational purposes, without warranty or restrictions.
// Compliments of:
//    Michael A. Covington, Ph.D., Consultant
//    Covington Innovations
//    http://www.covingtoninnovations.com/michael

// Key idea:
//
// The purpose of the ViewModel paradigm is to restrict interaction between
// the user interface (MainWindow, the "view") and the computation engine 
// (the "model" in older nomenclature).
//
// This file (Computation.cs) is identical in the WPF and WinForms version
// of the demonstration, except for some names.
//
// This is an example of how to apply the ViewModel concept to scientific 
// software, which typically invites the user to set up and launch a time-
// consuming computation, which runs in a separate thread and reports
// intermediate results as it goes.
//
// Screen output from such a program can take two forms: immediate updating of
// visible controls (in this case, the displayed countdown textbox), and writing
// successive messages to a "log," a textbox that is treated like an output file
// and never read back in.  This program demonstrates both.

// See also the comments in the other .cs files.


namespace VMdemoWPF
{
    
    class Computation
    {
        /// <summary>
        /// Count down, in seconds.
        /// This is an example of a time-consuming computation that
        /// reports intermediate results by assigning them to a string variable
        /// (which calls the setter of the property to which the variable refers).
        /// </summary>
        /// <param name="vm">
        /// The entire ViewModel, through which this
        /// method communicates with the user interface.
        /// </param>
        public static void Countdown(MainWindowViewModel vm)
        {
            // Not a strictly accurate countdown because the time taken to
            // update the display is not accounted for.
            int seconds = int.Parse(vm.SecondsToCount);

            vm.AppendToLog("Starting...\r\n");
            while (seconds > 0 && !vm.Canceler.IsCancellationRequested)
            {
                vm.DisplayedCountdown = seconds.ToString();
                seconds--;
                System.Threading.Thread.Sleep(1000);
            }
            
            vm.DisplayedCountdown = "0";
            if (vm.Canceler.IsCancellationRequested)
            {
                vm.AppendToLog("Cancelled.\r\n");
            }
            else
            {
                vm.AppendToLog("Finished.\r\n");
            }
        }
    }
}
